@@ -38,7 +38,7 @@ var have_webp = null;
 	img.onerror = function () {
 		have_webp = false;
 	};
-	img.src = "data:image/webp;base64,UklGRiIAAABXRUJQVlA4IBYAAAAwAQCdASoBAAEADsD+JaQAA3AAAAAA";
+	img.src = "data:image/webp;base64,UklGRhoAAABXRUJQVlA4TA0AAAAvAAAAEAcQERGIiP4HAA==";
 })();
 
 
@@ -48,7 +48,6 @@ function MPlayer() {
 	this.au = null;
 	this.au_native = null;
 	this.au_ogvjs = null;
-	this.cover_url = '';
 	this.tracks = {};
 	this.order = [];
 
@@ -858,14 +857,15 @@ var thegrid = (function () {
 				href = esc(ao.getAttribute('href')),
 				ref = ao.getAttribute('id'),
 				isdir = href.split('?')[0].slice(-1)[0] == '/',
+				ac = isdir ? ' class="dir"' : '',
 				ihref = href;
 
-			if (isdir) {
-				ihref = '/.cpr/ico/folder'
-			}
-			else if (r.thumbs) {
+			if (r.thumbs) {
 				ihref += (ihref.indexOf('?') === -1 ? '?' : '&') + 'th=' + (have_webp ? 'w' : 'j');
 			}
+			else if (isdir) {
+				ihref = '/.cpr/ico/folder';
+			}
 			else {
 				var ar = href.split('?')[0].split('.');
 				if (ar.length > 1)
@@ -886,7 +886,7 @@ var thegrid = (function () {
 			}
 
 			html.push('<a href="' + href + '" ref="' + ref + '"><img src="' +
-				ihref + '" /><span>' + ao.innerHTML + '</span></a>');
+				ihref + '" /><span' + ac + '>' + ao.innerHTML + '</span></a>');
 		}
 		lfiles.style.display = 'none';
 		gfiles.style.display = 'block';
